<?php
// early alpha     kner 2015

/* what i wanted was a number of individual 
 * electronic circuits - solutions included - for my classroom
 * 
 * what i made: use LTSpice vector graphic circuit, convert it to svg, replace the values
 * and orientation of current and voltage sources and resistors, simulate this new
 * circuit and present results as svg graphics
 * 
 *  
 * works for current, voltage, res
 * to shuffle orientation, the symbols must be symmetric related to (0,0) 
 * 
 * 
 * 
 * ToDo: test with more parts; import ARC and other constructs
 * 
 * 
 */ 

// S E T U P 

$schematicFilename = "x.asc";
$InputDirectory = "/home/roland/Dropbox/tmp/";
$OutputDirectory = "/tmp/";

$LIB = "/home/roland/.wine/drive_c/Programme/LTC/LTspiceIV/lib/sym/";
$SCAD3EXE='env WINEPREFIX="/home/roland/.wine"  wine /home/roland/.wine/drive_c/Programme/LTC/LTspiceIV/scad3.exe'; 
$imageMagicConvert="/usr/bin/convert";  //to produce pdf

//error_reporting(E_ALL ^ E_NOTICE);


if (!file_exists($InputDirectory)){
    mkdir($InputDirectory) or die("mkdir $InputDirectory not possible");
}

$schematic = array();
$pageWidth = 1024;
$pageHeight = 800;
$extremeLeft;
$extremeTop;
$extremeRight;
$extremeBottom;
$originalSources = array();


$MAXNUMBEROFSHEETS = 6;

for ($j = 0; $j < 3; $j++) {
    readSchematic($InputDirectory.$schematicFilename, $schematic);
    readSymbols($schematic, $symbols);
    insertMissingInfo($schematic, $symbols);

    shuffleSchematic($schematic);

    for ($i = 0; $i < $MAXNUMBEROFSHEETS; $i++) {

        $finished = false;
        helmholtz($schematic, $i, $finished);
        if ($finished)
            break;
        readSymbols($schematic, $symbols);
        $newSchematicFileName = "schematic.asc";
        $netlistFileName = 'schematic.net'; 
        WriteNewSchematic($schematic, $symbols, $InputDirectory . $newSchematicFileName, $i);
        createNetlistAndCombineWithSchematic($newSchematicFileName, $schematic);
        //simulate it
        shell_exec("cd $InputDirectory &&".$SCAD3EXE.' --ascii -b '. $netlistFileName);
        $simulationresults = array();
        $simulationresults = enterSimulationResults($schematic);
        require_once "./svglib/svglib.php";
        $svg1 = SVGDocument::getInstance();
        drawSVG($svgA, $pageWidth, $pageHeight, 0, 0, $schematic, $symbols, "original", $i);
        $svg1->setWidth($pageWidth);
        $pHeight = $extremeBottom - $extremeTop + 50;
        $svg1->setHeight(2 * $pageHeight);
        drawSVG($svgB, $pageWidth, $pageHeight, 0, $pageHeight, $schematic, $symbols, "values", $i);
        $svg1->addShape($svgA);
        $svg1->addShape($svgB);
        $svg1->addScript("style");
        //$svg1->output(); #output to browser, with header
        $svg1->asXML($OutputDirectory."svg$i.svg"); //output to svg file
    }
    exec($imageMagicConvert." ".$OutputDirectory."svg*.svg $OutputDirectory/svg$j.pdf");
}

function findValue($simulationresults, $name) {
    foreach ($simulationresults as $val) {
        if ($val["name"] == "V($name)") {
            return $val["value"];
        }
    }
}

function drawSVG(&$svg, $pageWidth, $pageHeight, $deltaX, $deltaY, $schematic, $symbols, $mode, $nr) {
    global $extremeLeft;
    global $extremeTop;
    global $extremeRight;
    global $extremeBottom;
    global $LIB;
    global $simulationresults;
///////////////////////////////////////////////////////////////////////////////////

    $svg = SVGGroup::getInstance("group1");

    $styl2 = new SVGStyle(array('fill' => 'none', 'stroke' => 'gray', 'stroke-width' => '3'));
    $styl1 = new SVGStyle(array('fill' => 'none', 'stroke' => 'gray', 'stroke-width' => '1'));
    $styl = new SVGStyle(array('fill' => 'blue', 'stroke' => 'gray', 'stroke-width' => '2'));
    $smallFont = new SVGStyle(array('font-size' => '8px'));
    //$normalFont = new SVGStyle(array('class' => 'normalfont'));
    $normalFontHidden = new SVGStyle(array('class' => 'normalfont_hidden'));


    foreach ($schematic["WIRE"] as $key => $val) {
        addline($svg, $val, 0, 0, 0, 0, $styl);
    }
    foreach ($schematic["FLAG"] as $key => $val) {
        $name = trim($val[3]);
        $x1 = intval($val[1]);
        $y1 = intval($val[2]);
        if ($name != "0") {
            $voltage = formatFloat(findValue($simulationresults, $name));
            $text = ($mode == "original") ? $name : $voltage;
            $txt = SVGText::getInstance($x1, $y1 - 2, "f" . $key, $text, "");
            $svg->addShape($txt);
        } else {
            //Masse
            $lineMasse = SVGLine::getInstance($x1 - 10, $y1, $x1 + 10, $y1, $id = "lm" . $key, $style = $styl2);
            $svg->addShape($lineMasse); #add the rect to svg
        }
    }
    foreach ($schematic["ENDPOINTS"] as $key => $val) {
        if ($val >= 3) {
            $xy = explode("|", $key);
            $x = intval($xy[0]);
            $y = intval($xy[1]);
            $c = SVGCircle::getInstance($x, $y, 3, $id = "c", $style = $styl);
            $svg->addShape($c);
        }
    }

    if (isset($schematic["TEXT"])) {
        foreach ($schematic["TEXT"] as $key => $val) {
            $txt = "";
            for ($i = 5; $i < count($val); $i++)
                $txt .= " " . $val[$i];
            $x2 = intval($val[1]);
            $y2 = intval($val[2]);
            addText($svg, $x2, $y2, 0, 0, 0, $txt, "");
        }
    }
    foreach ($schematic["SYMBOL"] as $key => $val) {
        $offsetx = intval($val[2]);
        $offsety = intval($val[3]);
        $angle = intval(substr($val[4], 1));
        $sym = $symbols[$LIB . $val[1]];

        // Symbole zeichnen

        foreach ($sym["LINE"] as $lineVal) {
            addline($svg, $lineVal, $offsetx, $offsety, $angle, 1, $styl);
        }
        if (isset($sym["CIRCLE"]))
            foreach ($sym["CIRCLE"] as $cVal) {
                $offsetx = intval($val[2]);
                $offsety = intval($val[3]);

                addCircle($svg, $cVal, $offsetx, $offsety, $angle, $styl1);
            }

        foreach ($sym["PIN"] as $pVal) {
            $x = intval($pVal[1]);
            $y = intval($pVal[2]);
            $order = $pVal["PINATTR"][2];
            $pin[$order]["x"] = $x;
            $pin[$order]["y"] = $y;
            if ($order == "1") {
                addText($svg, $x, $y + 2, $offsetx, $offsety, $angle, "*", $smallFont);
            }
        }
//        $x1 = $pin[1]["x"];
//        $y1 = $pin[1]["y"];
//        $x2 = $pin[2]["x"];
//        $y2 = $pin[2]["y"];
        // WINDOW (Schreibposition) setzen; Wenn kein WINDOW im Schematic, dann aus dem Symbol holen
        //$symbolname = $val[1];
        //$fn = str_replace("\ ", "\x20", $LIB . $symbolname);
        $attributWindowInstaName = $val["WINDOW"][0]["InstName"];
        $current = formatFloat($val["CURRENT"]);
        $current = $current . "A";
        $attributWindowInstaNameX = $val["WINDOW"][0]["x"];
        $attributWindowInstaNameY = $val["WINDOW"][0]["y"];
        $val2 = ($mode == "original") ? $attributWindowInstaName : $current;
        addText($svg, $attributWindowInstaNameX, $attributWindowInstaNameY, $offsetx, $offsety, $angle, $val2, $normalFontHidden);
        // addText($svg,$attributWindowInstaNameX,$attributWindowInstaNameY, $offsetx, $offsety, $angle,$attributWindowInstaName,$normalFont);
        $value = $val["WINDOW"][3]["Value"];
        if (!isset($val["NODES"]["NODE1"]["VALUE"])) {
            $val["NODES"]["NODE1"]["VALUE"] = 0;
        }
        if (!isset($val["NODES"]["NODE2"]["VALUE"])) {
            $val["NODES"]["NODE2"]["VALUE"] = 0;
        }
        $voltage = formatFloat($val["NODES"]["NODE1"]["VALUE"] - $val["NODES"]["NODE2"]["VALUE"]);
        $voltage = $voltage . "V";
        $attributWindowValueX = $val["WINDOW"][3]["x"];
        $attributWindowValueY = $val["WINDOW"][3]["y"];
        $val1 = ($mode == "original") ? $value : $voltage;
        addText($svg, $attributWindowValueX, $attributWindowValueY, $offsetx, $offsety, $angle, $val1, $normalFontHidden);
        //addText($svg,$attributWindowValueX,$attributWindowValueY, $offsetx, $offsety, $angle,$value  ,$normalFont);
    }

    $pagelength = $extremeRight - $extremeLeft + 100;
    $pageheight = $extremeBottom - $extremeTop + 100;
    $scaleX = $pageWidth / $pagelength;
    $scaleY = $pageHeight / $pageheight;

    $dX = $deltaX * $scaleX - $extremeLeft * $scaleX + 50;
    $dY = $deltaY - $extremeTop * $scaleX + 50;

    $txt = SVGText::getInstance($extremeLeft + 10, $extremeTop + 10, "nr", "Nr.:" . ($nr + 1), "styl2");
    $svg->addShape($txt);



    $svg->setAttribute('transform', "translate($dX,$dY) scale($scaleX)");
}

function checkExtremPoints($x, $y) {
    global $extremeLeft;
    global $extremeTop;
    global $extremeRight;
    global $extremeBottom;


    if (!isset($extremeLeft))
        $extremeLeft = $x;
    if (!isset($extremeRight))
        $extremeRight = $x;
    if (!isset($extremeTop))
        $extremeTop = $y;
    if (!isset($extremeBottom))
        $extremeBottom = $y;

    if ($x < $extremeLeft)
        $extremeLeft = $x;
    if ($y < $extremeTop) {
        $extremeTop = $y;
    }
    if ($x > $extremeRight)
        $extremeRight = $x;
    if ($y > $extremeBottom) {
        $extremeBottom = $y;
    }
}

function addline(&$svg, $val, $ox, $oy, $angle, $idx, $styl) {

    $x1 = $ox + intval($val[1 + $idx]);
    $y1 = $oy + intval($val[2 + $idx]);
    $x2 = $ox + intval($val[3 + $idx]);
    $y2 = $oy + intval($val[4 + $idx]);
    $line = SVGLine::getInstance($x1, $y1, $x2, $y2, $id = "l", $style = $styl);
    checkExtremPoints($x1, $y1);
    checkExtremPoints($x2, $y2);
    if ($idx == 1) {
        $line->rotate($angle, $ox, $oy);
    }
    $svg->addShape($line); #add the rect to svg
}

function addCircle(&$svg, $val, $ox, $oy, $angle, $styl) {
    $x1 = intval($val[2]);
    $y1 = intval($val[3]);
    $x2 = intval($val[4]);
    $y2 = intval($val[5]);

    $circ = SVGCircle::getInstance(($x1 + $x2) / 2 + $ox, ($y1 + $y2) / 2 + $oy, ($y2 - $y1) / 2, $id = "c", $style = $styl);
    $circ->rotate($angle, $ox, $oy);
    $svg->addShape($circ);
}

$id = 0;

function addText(&$svg, $x, $y, $ox, $oy, $angle, $text, $style) {
    global $id;
    global $styl1;
    if (empty($style))
        $style = $styl1;
    // Radius für Rotation

    $r = sqrt(pow($x, 2) + pow($y, 2));
    $ang = GetDegree($x, $y); // 4 Quadrant

    $xx = $r * cos(($ang + $angle) * M_PI / 180.0);
    $yy = $r * sin(($ang + $angle) * M_PI / 180.0);



    $txt = SVGText::getInstance($xx + $ox, $yy + $oy, "t", $text, $style = $style);
    //$a= $txt->rotate( $ox, $oy);
//    $svg->addShape($txt);
//    $txt->rotate(180, $ox, $oy);
    $txt->addAttribute("class", "red");
    $txt->addOnClick("this.style");
    $svg->addShape($txt);
}

function GetDegree($x, $y) {
    // we don't want to cause division by zero
    if ($x == 0)
        $x = 1 / 10000;

    $deg = rad2deg(atan(abs($y / $x)));

    if ($y >= 0)
        $deg = $x < 0 ? 180 - $deg : $deg;
    else
        $deg = $x < 0 ? 180 + $deg : 360 - $deg;

    return $deg;
}

function formatFloat($val) {
    $e = "";
    $x = abs(floatval($val));
    if ($x < 1E-9)
        $x = 0;
    else if (($x < 1E-6) && ($x >= 1E-9)) {
        $x = ($x * 1E9);
        $e = "n";
    } else if (($x < 1E-3) && ($x >= 1E-6)) {
        $x = ($x * 1E6);
        $e = "u";
    } else if (($x < 1) && ($x >= 1E-3)) {
        $x = ($x * 1000);
        $e = "m";
    } else if (($x < 1E3) && ($x >= 1E-0)) {
        $x = ($x * 1);
        $e = " ";
    } else if (($x >= 1E-3)) {
        $x = ($x / 1000);
        $e = "k";
    }

    $sign = $val < 0 ? -1 : 1;
    $x = round($x * 10) / 10;

    return ($sign * $x) . $e;
}

//Trim Netlist
/* LTSpice schreibt, wenn default, die WINDOW Properties nicht in die Netzliste
 * ==> Netzliste neu erzeugen um veränderte Bauteilwerte schreiben zu können
 */
function line($x, $count) {
    $s = "";
    $nr = 0;
    foreach ($x as $val) {
        $s .= $val . " ";
        if ($nr > $count - 2)
            break;
        $nr++;
    }
    $s .= "\n";
    return $s;
}

function WriteNewSchematic($schematic, $symbols, $schematicFileName, $nr) {
    global $LIB;
    global $extremeLeft;
    global $extremeBottom;

    $f = fopen($schematicFileName, "w") or die("Unable to write schematic $schematicFileName");
    fwrite($f, "Version 4\n");
    fwrite($f, line($schematic["SHEET"], 4));
    foreach ($schematic["WIRE"] as $val)
        fwrite($f, line($val, 5));
    foreach ($schematic["FLAG"] as $val)
        fwrite($f, line($val, 4));
//foreach($schematic["TEXT"] as $val) fwrite($f,line($val)); 
    foreach ($schematic["SYMBOL"] as $val) {
        $window = $val["WINDOW"];
        unset($val["WINDOW"]);
        fwrite($f, line($val, 5));
        fwrite($f, "WINDOW 0 ");
        fwrite($f, $window[0]["x"] . " ");
        fwrite($f, $window[0]["y"] . " ");
        fwrite($f, $window[0]["direction"] . " ");
        fwrite($f, $window[0]["param"] . "\n");
        fwrite($f, "WINDOW 3 ");
        fwrite($f, $window[3]["x"] . " ");
        fwrite($f, $window[3]["y"] . " ");
        fwrite($f, $window[3]["direction"] . " ");
        fwrite($f, $window[3]["param"] . "\n");
        fwrite($f, "SYMATTR InstName " . $window[0]["InstName"] . "\n");

        fwrite($f, "SYMATTR Value " . $window[3]["Value"] . " \n");
    }

    $l = $extremeLeft + 50;
    fwrite($f, "TEXT $extremeLeft $extremeBottom Left 2 !.op \n");
    fwrite($f, "TEXT $l $extremeBottom Left 2 $nr");


    fclose($f);
}
/*
 * 
 * Problem: wenn im Schaltplan die Default-Werte (Attribut VALUE) der Bauteile verwendet 
 * werden, so werden diese Werte nicht in den Schaltplan .asc eingetragen, sondern 
 * bei Bedarf aus der ibliothek gelesen. Vorteil: man kann mit einer Änderung alle Bauteil-
 * werte abändern Nachteil: will man den Schaltplan .asc manipulieren, dann fehlen diese 
 * Einträge ==> Netzliste umschreiben, so dass alle Attribute eingetragen sind. 
 * 
 * alle Texte werden entfernt und ein .op eingefügts
 * 
 
 */
function readSchematic($schematicfn, &$schematic) {
    $schematic = array(); 
    $myfile = fopen($schematicfn, "r") or die("Unable to open schematic!");
    $status = 0;
    $symbolcount = 0;
    while (!feof($myfile)) {
        $line = trim(fgets($myfile));
        if ($line == false)
            continue;
        $line = str_replace('\ ', '/x', $line);
        $line = str_replace('\\\\', '/', $line);
        $tokens = explode(' ', $line);
        $tokens[1] = str_replace('/x', '\ ', $tokens[1]);

        switch ($tokens[0]) {
            case "SHEET":

                $schematic["SHEET"] = $tokens;
                break;
            case "WIRE":
                $x1 = intval($tokens[1]);
                $y1 = intval($tokens[2]);
                $x2 = intval($tokens[3]);
                $y2 = intval($tokens[4]);
                if (empty($schematic["ENDPOINTS"][$x1 . "|" . $y1]))
                    $schematic["ENDPOINTS"][$x1 . "|" . $y1] = 1;
                else
                    $schematic["ENDPOINTS"][$x1 . "|" . $y1] ++;
                if (empty($schematic["ENDPOINTS"][$x2 . "|" . $y2]))
                    $schematic["ENDPOINTS"][$x2 . "|" . $y2] = 1;
                else
                    $schematic["ENDPOINTS"][$x2 . "|" . $y2] ++;
                $schematic["WIRE"][] = $tokens;
                break;
            case "FLAG":
                $schematic["FLAG"][] = $tokens;
                break;
            case "DATAFLAG":
                $schematic["DATAFLAG"][] = $tokens;
                break;
            case "TEXT":
                $schematic["TEXT"][] = $tokens;
                break;
            case "SYMBOL":
                $schematic["SYMBOL"][$symbolcount] = $tokens;

                $symbolcount++;
                break;
            case "SYMATTR":
                //$schematic["SYMBOL"][$symbolcount - 1]["SYMATTR"][] = $tokens;
                $type = $tokens[1];
                if ($type == "Value")
                    $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][3]["Value"] = $tokens[2];
                if ($type == "InstName")
                    $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][0]["InstName"] = $tokens[2];
                break;
            case "WINDOW":
                $id = intval($tokens[1]);
                $x1 = intval($tokens[2]);
                $y1 = intval($tokens[3]);

                $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][$id]["x"] = $x1;
                $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][$id]["y"] = $y1;
                $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][$id]["direction"] = $tokens[4];
                $schematic["SYMBOL"][$symbolcount - 1]["WINDOW"][$id]["param"] = $tokens[5];
                break;
        }
    }
    fclose($myfile);
}

function readSymbols(&$schematic, &$symbols) {
    global $LIB;
    foreach ($schematic["SYMBOL"] as $key => $val) {
        $windowcount = 0;
        $pincount = 0;
        $fn = str_replace("\ ", "\x20", $LIB . $val[1]);
        $myfile = fopen(realpath($fn . ".asy"), "r") or die("Unable to open symbol file!" . $fn);
        while (!feof($myfile)) {
            $line = trim(fgets($myfile));
            $tokens = explode(' ', $line);
            switch ($tokens[0]) {
                case "LINE":
                    $symbols[$fn]["LINE"][] = $tokens;
                    break;
                case "CIRCLE":
                    $symbols[$fn]["CIRCLE"][] = $tokens;
                    break;
                case "WINDOW":
                    $id = intval($tokens[1]);
                    $x1 = intval($tokens[2]);
                    $y1 = intval($tokens[3]);
                    $direction = ($tokens[4]);
                    $param = ($tokens[5]);
                    $symbols[$fn]["WINDOW"][$id]["x"] = $x1;
                    $symbols[$fn]["WINDOW"][$id]["y"] = $y1;
                    $symbols[$fn]["WINDOW"][$id]["direction"] = $direction;
                    $symbols[$fn]["WINDOW"][$id]["param"] = $param;
                    break;
                case "SYMATTR":
                    $type = $tokens[1];
                    if ($type == "Value")
                        $symbols[$fn]["WINDOW"][3]["Value"] = $tokens[2];
                    if ($type == "Prefix")
                        $symbols[$fn]["WINDOW"][0]["Prefix"] = $tokens[2];
                    break;
                case "PIN":
                    $symbols[$fn]["PIN"][$pincount++] = $tokens;
                    break;
                case "PINATTR":
                    $symbols[$fn]["PIN"][$pincount - 1]["PINATTR"] = $tokens;
                    break;
            }
        }
        fclose($myfile);
    }
}

////////////////////////////////////////////////////////////////////////////////
// Insert missing Window and Pinattrib entries into schematic .asc            //
////////////////////////////////////////////////////////////////////////////////
function insertMissingInfo(&$schematic, &$symbols) {
    global $LIB;
    foreach ($schematic["SYMBOL"] as $key => $val) {
        $symbolname = str_replace("\ ", "\x20", $LIB . $val[1]);
        $value0 = $symbols[$symbolname]["WINDOW"][0];
        if (!isset($val["WINDOW"][0]["x"])) {
            $schematic["SYMBOL"][$key]["WINDOW"][0]["x"] = $value0["x"];
            $schematic["SYMBOL"][$key]["WINDOW"][0]["y"] = $value0["y"];
            $schematic["SYMBOL"][$key]["WINDOW"][0]["direction"] = $value0["direction"];
            $schematic["SYMBOL"][$key]["WINDOW"][0]["param"] = $value0["param"];
        }
        $value3 = $symbols[$symbolname]["WINDOW"][3];
        if (!isset($val["WINDOW"][3]["Value"])) {
            $schematic["SYMBOL"][$key]["WINDOW"][3]["Value"] = $value3["Value"];
        }
        if (!isset($schematic["SYMBOL"][$key]["WINDOW"][3]["x"])) {
            $schematic["SYMBOL"][$key]["WINDOW"][3]["x"] = $value3["x"];
            $schematic["SYMBOL"][$key]["WINDOW"][3]["y"] = $value3["y"];
            $schematic["SYMBOL"][$key]["WINDOW"][3]["direction"] = $value3["direction"];
            $schematic["SYMBOL"][$key]["WINDOW"][3]["param"] = $value3["param"];
        }
    }
}

function enterSimulationResults(&$schematic) {
    global $InputDirectory;
    $rawfn = $InputDirectory . "schematic.raw";

///// Read RAW Simulation Results
    $myRaw = fopen($rawfn, "r") or die("Unable to open rawfile!");
    $results = array();
    $vars = false;
    $vals = false;
    while (!feof($myRaw)) {
        $line = strtoupper(fgets($myRaw));

        $items = preg_split("/[\s]+/", $line);
        if (substr($items[0], 0, 10) == "VARIABLES:") {
            $vars = true;
            continue;
        }
        if (!$vars & !$vals)
            continue;
        if ($vars) {
            $results[$items[1]]["name"] = $items[2];
            $results[$items[1]]["type"] = $items[3];
        }
        if (substr($items[0], 0, 7) == "VALUES:") {
            $vals = true;
            $vars = false;
            $valNr = 0;
            continue;
        }
        if (!$vals)
            continue;
        if ($vals) {
            $results[$valNr++]["value"] = $items[1];
        }
    }

// Enter results into schematic
    foreach ($results as $resultVal) {
        $name = $resultVal["name"];
        $nodeName = substr($name, 2, strlen($name) - 3);
        if ($resultVal["type"] == "VOLTAGE") {
            //search schematic for this node and enter its value
            foreach ($schematic["SYMBOL"] as $symbKey => $symb) {
                foreach ($symb["NODES"] as $nodekey => $val) {
                    if ($val["NAME"] == $nodeName) {
                        $res = $resultVal["value"];
                        $schematic["SYMBOL"][$symbKey]["NODES"][$nodekey]["VALUE"] = $res;
                    }
                }
            }
        } else {
            //device_current    
            foreach ($schematic["SYMBOL"] as $symbKey => $symb) {
                if ($symb["WINDOW"][0]["InstName"] == "$nodeName") {
                    $schematic["SYMBOL"][$symbKey]["CURRENT"] = $resultVal["value"];
                }
            }
        }
    }

    return $results;
}

function createNetlistAndCombineWithSchematic($schematicFileName, &$schematic) {
    global $InputDirectory;
    global $SCAD3EXE; 
    $netlistfn = $InputDirectory . "schematic.net";
    $x = shell_exec($SCAD3EXE.' -netlist  ' . $InputDirectory . $schematicFileName);
// read netlist
    $myNetlist = fopen($netlistfn, "r") or die("Unable to open netlist!");
    $netlist = array();
    while (!feof($myNetlist)) {
        $line = trim(fgets($myNetlist));
        $items = explode(" ", $line);

        $netlist[$items[0]] = $items;
    }

    $lines = file($netlistfn);
    $nl = fopen($netlistfn, "w") or die("Unable to open netlist for writing!");
    foreach ($lines as $val1) {
        if ($val1 == ".backanno\xD\n") {
            fwrite($nl, ".op\n");
        } else {
            fwrite($nl, $val1);
        }
    }
    fclose($nl);
// Combine Netlist and Schematic
    foreach ($schematic["SYMBOL"] as $key => $val) {
        $instname = $schematic["SYMBOL"][$key]["WINDOW"][0]["InstName"];
        $schematic["SYMBOL"][$key]["NODES"]["NODE1"]["NAME"] = $netlist[$instname][1];
        $schematic["SYMBOL"][$key]["NODES"]["NODE2"]["NAME"] = $netlist[$instname][2];
    }
}

function shuffleSchematic(&$schematic) {
    foreach ($schematic["SYMBOL"] as $key => $symb) {

        if ($symb[1] == "res") {
            $val = round((rand(2, 10) * 1000) / 1000) * 1000;
            $schematic["SYMBOL"][$key]["WINDOW"][3]["Value"] = formatFloat($val);
        }

        if ($symb[1] == "voltage") {
            $val = round((rand(1, 10)) / 1) * 1;
            $schematic["SYMBOL"][$key]["WINDOW"][3]["Value"] = formatFloat($val) . "V";
        }
        if ($symb[1] == "current") {
            $val = round((rand(1, 10) / 1000) * 1000) / 1000;
            $schematic["SYMBOL"][$key]["WINDOW"][3]["Value"] = formatFloat($val) . "A";
        }
        $orientation = $symb[4];
        $new_orientation = "R0";
        if (($orientation == "R0") || ($orientation == "R180")) {
            $new_orientation = (rand(0, 1) == 1) ? "R0" : "R180";
        }
        if (($orientation == "R90") || ($orientation == "R270")) {
            $new_orientation = (rand(0, 1) == 1) ? "R90" : "R270";
        }
        $schematic["SYMBOL"][$key][4] = $new_orientation;

//        $type = $symb[1];
//        if ($type == "voltage") {
//            $new_type = (rand(0, 1) == 1) ? "voltage" : "voltage_inactive";
//            $schematic["SYMBOL"][$key][1] = $new_type;
//        }        
//        if ($type == "current") {
//            $new_type = (rand(0, 1) == 1) ? "current" : "current_inactive";
//            $schematic["SYMBOL"][$key][1] = $new_type;
//        }
    }
}

function findSources($symbols) {
    $found = array();
    $nr = 0;
    foreach ($symbols as $key => $val) {
        if (($val[1] == "voltage") || ($val[1] == "current")) {
            $found[$nr]["index"] = $key;
            $found[$nr]["type"] = $val["1"];
            $found[$nr]["value"] = $val["WINDOW"][3]["Value"];
            $nr++;
        }
    }
    return $found;
}

function helmholtz(&$schematic, $nr, &$finished) {
    // Nummer 0 ==> Original
    // Nummer 1 ==> nur Quelle 1 aktiv usw.      
    global $originalSources;
    if ($nr == 0) {
        $originalSources = findSources($schematic["SYMBOL"]);
        return;
    }
    if (!isset($originalSources[$nr - 1])) {
        $finished = true;
        return;
    }
    foreach ($originalSources as $symbolnr => $symbolval) {
        $type = $symbolval["type"];
        if ($type == "voltage") {
            $new_type = (($nr - 1) == $symbolnr) ? "voltage" : "voltage_inactive";
            $schematic["SYMBOL"][$symbolval["index"]][1] = $new_type;
            $new_value = (($nr - 1) == $symbolnr) ? $symbolval["value"] : 0;
            $schematic["SYMBOL"][$symbolval["index"]]["WINDOW"][3]["Value"] = $new_value;
        }
        if ($type == "current") {
            $new_type = (($nr - 1) == $symbolnr) ? "current" : "current_inactive";
            $schematic["SYMBOL"][$symbolval["index"]][1] = $new_type;
            $new_value = (($nr - 1) == $symbolnr) ? $symbolval["value"] : 0;
            $schematic["SYMBOL"][$symbolval["index"]]["WINDOW"][3]["Value"] = $new_value;
        }
    }
}



?>

Create Circuit and Symbol Files and put them into the proper Dirs

Symbols are special: they are symmetric with respect to the Origin - so it is 
possible to rotate them randomly ...
<pre>

#TEST CIRCUIT
Version 4
SHEET 1 880 680
WIRE 608 -16 -96 -16
WIRE -16 48 -32 48
WIRE 48 48 -16 48
WIRE 240 48 48 48
WIRE 256 48 240 48
WIRE 288 48 256 48
WIRE 400 48 384 48
WIRE 496 48 400 48
WIRE 528 48 496 48
WIRE -32 112 -32 48
WIRE 256 112 256 48
WIRE 48 128 48 48
WIRE 400 128 400 48
WIRE -32 256 -32 208
WIRE 48 256 48 224
WIRE 48 256 -32 256
WIRE 256 256 256 208
WIRE 256 256 48 256
WIRE 400 256 400 224
WIRE 400 256 256 256
WIRE 528 256 528 240
WIRE 528 256 400 256
WIRE -32 272 -32 256
WIRE -96 336 -96 -16
WIRE 608 336 608 -16
WIRE 608 336 -96 336
FLAG -32 272 0
DATAFLAG -16 48 ""
DATAFLAG 240 48 ""
DATAFLAG 496 48 ""
SYMBOL voltage 528 192 R0
WINDOW 0 27 -41 Left 2
SYMATTR InstName V1
SYMATTR Value 1V
SYMBOL res 336 48 R90
WINDOW 0 -18 -6 VBottom 2
WINDOW 3 18 -5 VTop 2
SYMATTR InstName R2
SYMBOL res -32 160 R180
WINDOW 0 22 9 Left 2
WINDOW 3 26 -19 Left 2
SYMATTR InstName R3
SYMBOL current 256 160 R180
WINDOW 0 11 50 Left 2
WINDOW 3 9 -51 Left 2
SYMATTR InstName I2
SYMATTR Value 1mA
SYMBOL current 48 176 R180
WINDOW 0 -39 -15 Left 2
WINDOW 3 -47 10 Left 2
SYMATTR InstName I3
SYMATTR Value 1mA
SYMBOL res 528 96 R0
WINDOW 0 21 -15 Left 2
WINDOW 3 20 15 Left 2
SYMATTR InstName R4
SYMBOL res 400 176 R0
WINDOW 0 31 -20 Left 2
WINDOW 3 32 5 Left 2
SYMATTR InstName R5
TEXT -80 296 Left 2 !.op

#VOLTAGE 
Version 4
SymbolType CELL
LINE Normal -23 -41 -7 -41
LINE Normal -15 -49 -15 -33
LINE Normal 0 48 0 40
LINE Normal 0 -48 0 -40
LINE Normal 0 -33 0 -40
LINE Normal 0 31 0 40
LINE Normal 0 31 0 -33
CIRCLE Normal -32 -33 32 31
WINDOW 0 31 -37 Left 2
WINDOW 3 24 39 Left 2
SYMATTR Value V
SYMATTR Prefix V
SYMATTR Description Voltage Source, either DC, AC, PULSE, SINE, PWL, EXP, or SFFM
PIN 0 -48 NONE 0
PINATTR PinName +
PINATTR SpiceOrder 1
PIN 0 48 NONE 0
PINATTR PinName -
PINATTR SpiceOrder 2



#VOLTAGE INACTIVE 
Version 4
SymbolType CELL
LINE Normal 0 -48 0 48
WINDOW 0 21 -47 Left 2
WINDOW 3 25 31 Left 2
SYMATTR Value 0
SYMATTR Prefix V
SYMATTR Description Voltage Source, either DC, AC, PULSE, SINE, PWL, EXP, or SFFM
PIN 0 -48 NONE 0
PINATTR PinName +
PINATTR SpiceOrder 1
PIN 0 48 NONE 0
PINATTR PinName -
PINATTR SpiceOrder 2

#CURRENT SOURCE
Version 4
SymbolType CELL
LINE Normal 0 48 0 40
LINE Normal 0 -48 0 -40
LINE Normal 0 -33 0 -40
LINE Normal 0 40 0 31
LINE Normal -32 0 32 0
LINE Normal 0 -7 0 -28
LINE Normal -4 -17 0 -7
LINE Normal 4 -17 -4 -17
LINE Normal 0 -7 4 -17
CIRCLE Normal -32 -33 32 31
WINDOW 0 17 -38 Left 2
WINDOW 3 23 30 Left 2
SYMATTR Value I
SYMATTR Prefix I
PIN 0 -48 NONE 0
PINATTR PinName +
PINATTR SpiceOrder 1
PIN 0 48 NONE 0
PINATTR PinName -
PINATTR SpiceOrder 2


#CURRENT INACTIVE
Version 4
SymbolType CELL
LINE Normal 0 48 0 32
LINE Normal 0 -48 0 -32
LINE Normal -23 0 25 0
WINDOW 0 24 -48 Left 2
WINDOW 3 25 32 Left 2
SYMATTR Value 0
SYMATTR Prefix I
SYMATTR Description inactive Current source 
PIN 0 -48 NONE 0
PINATTR PinName +
PINATTR SpiceOrder 1
PIN 0 48 NONE 0
PINATTR PinName -
PINATTR SpiceOrder 2



# RESISTOR
Version 4
SymbolType CELL
LINE Normal 0 32 0 48
LINE Normal 0 -48 0 -38
LINE Normal 11 -40 -11 -40
LINE Normal 11 -39 -11 -39
LINE Normal 11 -38 -11 -38
LINE Normal -11 32 -11 -40
LINE Normal 11 -40 11 32
LINE Normal 11 32 -11 32
WINDOW 0 17 -14 Left 2
WINDOW 3 17 5 Left 2
SYMATTR Value 1k
SYMATTR Prefix R
SYMATTR Description A Resistor(European style graphic)
PIN 0 -48 NONE 0
PINATTR PinName A
PINATTR SpiceOrder 1
PIN 0 48 NONE 0
PINATTR PinName B
PINATTR SpiceOrder 2

</pre>